import { guard as predicateguard } from './Predicate.js';
export function get(a, i) {
    if (guard(a) && typeof i === 'number') {
        if (i >= 0 && i < a.length) {
            return a[i];
        }
        return new RangeError(`Index out of bounds ${i} for length ${a.length}`);
    }
    return (b) => {
        if (a >= 0 && a < b.length) {
            return b[a];
        }
        return new RangeError(`Index out of bounds ${a} for length ${b.length}`);
    };
}
;
export function getOr(a, b, i) {
    if (arguments.length === 3 && Array.isArray(a) && typeof i === 'number') {
        if (i >= 0 && i < a.length) {
            return a[i];
        }
        return b;
    }
    return (b, d) => {
        if (d >= 0 && d < b.length) {
            return b[d];
        }
        return a;
    };
}
;
/**
 * Returns the first element of an array `A`. If the array is empty, returns `Option.None`.
 *
 * @example
 *
 * ```ts
 * import { Arr } 'tiinvo';
 *
 * Arr.first(['a', 'b']) // 'a';
 * Arr.first([])         // null;
 * ```
 *
 * @template A array's type
 * @param t the array
 * @returns
 *  - `Option.Some<A>` if some
 *  - `Option.None` otherwise
 * @group Accessors
 * @since 4.0.0
 */
export const first = (t) => t.length > 0 ? t[0] : null;
export function firstOr(t, b) {
    if (guard(t)) {
        return t.length > 0 ? t[0] : b;
    }
    return (b) => b.length > 0 ? b[0] : t;
}
/**
 * Returns the last element of an array `A`.
 *
 * @example
 *
 * ```ts
 * import { Arr } 'tiinvo';
 *
 * Arr.last(['a', 'b']) // 'b';
 * Arr.last([])         // null;
 * ```
 *
 * @template A array's type
 * @param t the array
 * @returns
 *  - `Option.Some<A>` if has a last element
 *  - `Option.None` otherwise
 * @group Accessors
 * @since 4.0.0
 */
export const last = (t) => (t[t.length - 1] ?? null);
export function lastOr(t, b) {
    if (guard(t)) {
        return t[t.length - 1] ?? b;
    }
    return (b) => b[b.length - 1] ?? t;
}
//#endregion
//#region guardables
/**
 * Returns true if `a` is an array.
 *
 * ```ts
 * import { Arr } from 'tiinvo';
 *
 * Arr.guard([])          // true
 * Arr.guard(null)        // false
 * Arr.guard(undefined)   // false
 * Arr.guard(0)           // false
 * Arr.guard('')          // false
 * ```
 *
 * @param x the value to check
 * @returns
 *  - true if x is an array
 *  - false otherwise
 * @group Guardables
 * @since 4.0.0
 */
export const guard = (x) => Array.isArray(x);
export function guardOf(g, x) {
    const c = (g, x) => {
        if (!guard(x)) {
            return false;
        }
        for (let i = 0; i < x.length; i++) {
            if (!g(x[i])) {
                return false;
            }
        }
        return true;
    };
    if (arguments.length === 2) {
        return c(g, x);
    }
    return (x) => c(g, x);
}
;
export function cmp(cmp, a, b) {
    const _cmp = (x, y) => {
        const c = typeof cmp === 'function' ? cmp : cmp.cmp;
        if (x.length > y.length) {
            return 1;
        }
        else if (x.length < y.length) {
            return -1;
        }
        let s = 0;
        for (let i = 0; i < x.length; i++) {
            s += c(x[i], y[i]);
        }
        if (s === 0) {
            return s;
        }
        else if (s >= 1) {
            return 1;
        }
        else {
            return -1;
        }
    };
    if (arguments.length === 3 && guard(a) && guard(b)) {
        return _cmp(a, b);
    }
    else if (arguments.length === 2 && guard(a)) {
        return (b) => _cmp(a, b);
    }
    else if (arguments.length === 1) {
        return (a, b) => _cmp(a, b);
    }
}
;
export function eq(e, a, b) {
    const _eq = (x, y) => {
        const _e = typeof e === 'function' ? e : e.eq;
        if (x.length !== y.length) {
            return false;
        }
        let s = 0;
        for (let i = 0; i < x.length; i++) {
            if (_e(x[i], y[i])) {
                continue;
            }
            else {
                return false;
            }
        }
        return true;
    };
    if (guard(a) && guard(b)) {
        return _eq(a, b);
    }
    else if (guard(a) && !guard(b)) {
        return (c) => _eq(a, c);
    }
    else {
        return (c, d) => _eq(c, d);
    }
}
;
export function concat(a, b) {
    if (guard(b) && guard(a)) {
        return a.concat(b);
    }
    return (b) => b.concat(a);
}
export function contains(a, b) {
    if (guard(a)) {
        return a.indexOf(b) >= 0;
    }
    return (b) => guard(b) && b.indexOf(a) >= 0;
}
export function every(a, p) {
    if (guard(a) && typeof p === 'function') {
        return a.every(p);
    }
    return (b) => b.every(a);
}
/**
 * Creates an array from an array-like object.
 *
 * ```ts
 * import { Arr } 'tiinvo';
 *
 * Arr.from([1, 2, 3]) // [1, 2, 3]
 * Arr.from(new Set([1, 2, 3])) // [1, 2, 3]
 * ```
 *
 * @param a an array-like object to convert to an array.
 * @returns the array
 * @group Native methods
 * @since 4.0.0
 */
export const from = Array.from;
export function fill(a, b, start, end) {
    if (guard(a)) {
        return a.fill(b, start, end);
    }
    return (c, start2, end2) => c.fill(b, b ?? start2, start ?? end2);
}
export function filter(a, p) {
    if (guard(a) && predicateguard(p)) {
        return a.filter(p);
    }
    return (b) => b.filter(a);
}
export function find(a, p) {
    if (guard(a) && predicateguard(p)) {
        return a.find(p) ?? null;
    }
    return (b) => b.find(a) ?? null;
}
export function filterMap(a, mod) {
    const gmod = (x) => typeof x === 'object' && x !== null && 'filter' in x && 'map' in x;
    if (guard(a) && gmod(mod)) {
        const r = [];
        for (let i = 0; i < a.length; i++) {
            const v = a[i];
            if (mod.filter(v)) {
                r.push(mod.map(v));
            }
        }
        return r;
    }
    else if (gmod(a) && !Array.isArray(a)) {
        return (b) => {
            const r = [];
            for (let i = 0; i < b.length; i++) {
                if (a.filter(b[i])) {
                    r.push(a.map(b[i]));
                }
            }
            return r;
        };
    }
    throw new Error('Invalid functor passed, required Functors.FilterReduceable<a, b>');
}
export function filterReduce(a, mod) {
    if (guard(a) && typeof mod === 'object' && mod !== null && 'filter' in mod && 'reduce' in mod && 'default' in mod) {
        let out = mod.default;
        for (let i = 0; i < a.length; i++) {
            if (mod.filter(a[i])) {
                out = mod.reduce(out, a[i]);
            }
        }
        return out;
    }
    else if (typeof a === 'object' && a !== null && 'filter' in a && 'reduce' in a && 'default' in a) {
        return (b) => {
            let out = a.default;
            for (let i = 0; i < b.length; i++) {
                if (a.filter(b[i])) {
                    out = a.reduce(out, b[i]);
                }
            }
            return out;
        };
    }
    else {
        throw new Error('Invalid functor passed, required Functors.FilterReduceable<a, b>');
    }
}
export function flat(a, d) {
    if (guard(a)) {
        return a.flat(d);
    }
    return (b) => b.flat(a);
}
export function flatMap(a, m) {
    if (guard(a) && typeof m === 'function') {
        return flat(a.map(b => b.map(m)), 1);
    }
    return (b) => flat(b.map(c => c.map(a)), 1);
}
export function includes(a, b) {
    if (guard(a) && arguments.length === 2) {
        return a.includes(b);
    }
    return (b) => b.includes(a);
}
/**
 * Gets the length of the array. This is a number one higher than the highest index in the array.
 *
 * @example
 *
 * ```ts
 * import { Arr } from 'tiinvo';
 *
 * Arr.length([])         // 0
 * Arr.length([1])        // 1
 * Arr.length([1, 2, 3])  // 3
 * ```
 *
 * @template a array's type
 * @param t the array
 * @returns the length of the array
 * @group Native methods
 * @since 4.0.0
 */
export const length = (t) => t.length;
export function join(a, b) {
    if (guard(a)) {
        return a.join(b ?? '');
    }
    return (b) => b.join(a ?? '');
}
export function make(size, d) {
    const a = [];
    for (let i = 0; i < size; i++) {
        if (typeof d === 'function') {
            a.push(d(i));
        }
        else {
            a.push(d);
        }
    }
    return a;
}
;
export function map(a, m) {
    if (guard(a)) {
        return a.map(m);
    }
    return (b) => b.map(a);
}
export function none(a, m) {
    if (guard(a) && predicateguard(m)) {
        return !a.some(m);
    }
    return (b) => !b.some(a);
}
/**
 * Returns a new array from a set of elements.
 *
 * ```typescript
 * import { Arr } 'tiinvo';
 *
 * Arr.of(1, 2, 3) // [1, 2, 3]
 * ```
 *
 * @param a
 *
 * @template T
 * @param items A set of elements to include in the new array object.
 * @returns the new array
 * @group Native methods
 * @since 4.0.0
 */
export const of = Array.of;
export function partition(a, f) {
    const _part = (_x, _f) => {
        const _o = [[], []];
        for (let i = 0; i < _x.length; i++) {
            if (_f(_x[i])) {
                _o[0].push(_x[i]);
            }
            else {
                _o[1].push(_x[i]);
            }
        }
        return _o;
    };
    if (guard(a) && typeof f === 'function') {
        return _part(a, f);
    }
    else if (typeof a === 'function') {
        return (c) => _part(c, a);
    }
}
/**
 * Returns a random element of an array `A`.
 *
 * If the array is empty, returns `Option.None`
 *
 * ```typescript
 * import { Arr } 'tiinvo';
 *
 * Arr.random(['a', 'b', 'c']) // 'a' or 'b' or 'c'
 * ```
 *
 * @template A array's type
 * @param a the array
 * @returns a random element of the array
 *  - `Option.Some<A>` if the array is not empty
 *  - `Option.None` if the array is empty
 * @group Misc
 * @since 4.0.0
 */
export const random = (a) => (a[Math.floor(Math.random() * a.length)]) ?? null;
export function reduce(a, r, b) {
    if (guard(a)) {
        return a.reduce(r, b);
    }
    return (c) => c.reduce(a, r);
}
export function reduceRight(a, r, b) {
    if (guard(a)) {
        return a.reduceRight(r, b);
    }
    return (c) => c.reduceRight(a, r);
}
/**
 * Reverses the elements in an array in place without mutating the original array.
 *
 * @example
 *
 * ```ts
 * import { Arr } 'tiinvo';
 *
 * Arr.reverse([10, 20, 30])      // [30, 20, 10]
 * ```
 *
 * @template A the array's type
 * @param a the array
 * @returns the reversed array
 * @group Native methods
 * @since 4.0.0
 */
export const reverse = (a) => a.reverse();
/**
 * Shuffles an array
 *
 * @example
 *
 * ```ts
 * import { Arr } from 'tiinvo'
 *
 * Arr.shuffle([10, 20, 30])  // could be [10, 30, 20] or [20, 30, 10] or [30, 20, 10] or ...
 * ```
 *
 * @template A the array's type
 * @param a the array
 * @returns the shuffled array
 * @group Misc
 * @since 4.0.0
 */
export const shuffle = (a) => {
    const b = [].slice.call(a);
    for (let i = b.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        [b[i], b[j]] = [b[j], b[i]];
    }
    return b;
};
export function slice(a, s, e) {
    if (guard(a)) {
        return a.slice(s, e);
    }
    return (b) => b.slice(a, s);
}
export function some(a, p) {
    if (guard(a) && predicateguard(p)) {
        return a.some(p);
    }
    return (b) => b.some(a);
}
export function sort(a, cmp) {
    if (guard(a)) {
        return a.sort(cmp);
    }
    return (b) => b.sort(a);
}
export function zip(a, b) {
    const _zip = (x, y) => {
        const _o = new Array(Math.min(x.length, y.length));
        for (let i = 0; i < _o.length; i++) {
            _o[i] = [
                x[i],
                y[i]
            ];
        }
        return _o;
    };
    if (guard(a) && guard(b)) {
        return _zip(a, b);
    }
    return (c) => _zip(c, a);
}
//#endregion
//#region predicates
/**
 * Returns `true` if the array `t<any>` is empty.
 *
 * ```ts
 * import { Arr } from 'tiinvo';
 *
 * Arr.empty([])     // true
 * Arr.empty(['a'])  // false
 * ```
 *
 * @param t the array
 * @returns
 *  - `true` if is empty
 *  - `false` otherwise
 * @group Predicates
 * @since 4.0.0
 */
export const empty = (t) => t.length === 0;
/**
 * Returns `true` if the array `t<any>` is populated.
 *
 * ```ts
 * import { Arr } from 'tiinvo';
 *
 * Arr.populated([])     // false
 * Arr.populated(['a'])  // true
 * ```
 *
 * @param t the array
 * @returns
 *  - `true` if the array is populated
 *  - `false` otherwise
 * @group Predicates
 * @since 4.0.0
 */
export const populated = (t) => t.length > 0;
//#endregion
