import type * as Functors from './Functors.js';
import type * as Fn from './Fn.js';
import type * as Result from './Result.js';
/**
 * Represents a very large integer.
 *
 * @since 4.0.0
 */
export type T = bigint;
/**
 * Checks if a parameter `x` is a `BigInt`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo'
 *
 * BigInt.guard(10)     // false
 * BigInt.guard(10n)    // true
 * ```
 *
 * @param x the parameter to check
 * @returns returns true if is a bigint, false otherwise
 * @group Guards
 * @since 4.0.0
 */
export declare const guard: (x: unknown) => x is bigint;
/**
 * Compares two `BigInt` `a` and `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo'
 *
 * BigInt.cmp(1n, 0n)       // 1
 * BigInt.cmp(0n, 0n)       // 0
 * BigInt.cmp(0n, 1n)       // -1
 * ```
 *
 * @param a the first bigint
 * @param a the second bigint
 * @returns
 * - -1 if `a` is less than `b`
 * - 0 if `a` is equal to `b`
 * - 1 if `a` is more than `b`
 * @group Comparables
 * @since 4.0.0
 */
export declare function cmp(a: T, b: T): Functors.ComparableResult;
/**
 * Returns a unary function which compares two `BigInt` `b` and `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo'
 *
 * const cmp0 = BigInt.cmp(0n)
 *
 * cmp0(0n)      // 0
 * cmp0(1n)      // -1
 * cmp0(-1n)     // 1
 * ```
 *
 * @param a the first bigint
 * @param a the second bigint
 * @returns the unary function
 * @group Comparables
 * @since 4.0.0
 */
export declare function cmp(a: T): Fn.Unary<T, Functors.ComparableResult>;
/**
 * Checks if `a` and `b` are equal
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo'
 *
 * BigInt.eq(10n, 10n)              // true
 * BigInt.eq(10n, 5n)               // false
 * BigInt.eq(10n, 10 as any)        // false
 * ```
 *
 * @param a the first bigint
 * @param b the second bigint
 * @returns true if two bigints are equal
 * @group Comparables
 * @since 4.0.0
 */
export declare function eq(a: T, b: T): boolean;
/**
 * Returns a unary function which checks if `b` and `a` are equal
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo'
 *
 * const is10n = BigInt.eq(10n)
 *
 * is10n(10n)              // true
 * is10n(5n)               // false
 * is10n(10 as any)        // false
 * ```
 *
 * @param a the first bigint
 * @returns the unary function
 * @group Comparables
 * @since 4.0.0
 */
export declare function eq(a: T): Fn.Unary<T, boolean>;
/**
 * Maps a bigint `a` to a value `Result.Ok<b>` if a is `bigint`,
 * otherwise returns `Result.Err`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const toHex = BigInt.map(x => '0x' + x.toString(16))
 *
 * BigInt.map(10n, x => '0x' + x.toString(16))       // "0xa"
 * BigInt.map("a", x => '0x' + x.toString(16))       // TypeError("a is not a bigint")
 * ```
 *
 * @template A the mapped value
 * @param a the bigint
 * @param m the mappable functor
 * @returns the mapped value or TypeError if `a` is not a `bigint`
 * @group Mappables
 * @since 4.0.0
 */
export declare function map<A>(a: T, m: Functors.Mappable<T, A>): Result.T<A>;
/**
 * Returns a unary function which maps a bigint `a` to
 * a value `Result.Ok<b>` if a is `bigint`, otherwise returns `Result.Err`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const toHex = BigInt.map(x => '0x' + x.toString(16))
 *
 * toHex(10n)       // "0xa"
 * toHex("a")       // TypeError("a is not a bigint")
 * ```
 *
 * @template A the mapped value
 * @param a the mappable functor
 * @returns the unary function
 * @group Mappables
 * @since 4.0.0
 */
export declare function map<A>(a: Functors.Mappable<T, A>): Fn.Unary<T, Result.T<A>>;
/**
 * Maps a bigint `a` to a value of type `B` if a is `bigint`, otherwise returns `b`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const toHex = BigInt.mapOr(x => '0x' + x.toString(16), "0x0")
 *
 * toHex(10n)      // "0xa"
 * toHex("a")      // "0x0"
 * ```
 *
 * @template A the mapped value type
 * @param a the bigint
 * @param m the mappable functor
 * @param b the fallback value
 * @group Mappables
 * @since 4.0.0
 */
export declare function mapOr<A>(a: T, m: Functors.Mappable<T, A>, b: A): A;
/**
 * Returns a unary function which maps a bigint `b` to a value of type `B` if a is `bigint`, otherwise returns `m`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const toHex = BigInt.mapOr(x => '0x' + x.toString(16), "0x0")
 *
 * toHex(10n)      // "0xa"
 * toHex("a")      // "0x0"
 * ```
 *
 * @template A the mapped value type
 * @param a the bigint
 * @param m the mappable functor
 * @returns the unary function
 * @group Mappables
 * @since 4.0.0
 */
export declare function mapOr<A>(a: Functors.Mappable<T, A>, m: A): Fn.Unary<T, A>;
/**
 * Adds `a` to `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.add(5n, -2n)             // 3n
 * BigInt.add(5n, 12n)             // 17n
 *
 * const add5 = BigInt.add(5n)
 *
 * add5(10n)                   // 15n
 * ```
 *
 * @param a - the first bigint
 * @param b - the second bigint
 * @returns the result
 * @group Operables
 * @since 4.0.0
 */
export declare function add(a: T, b: T): T;
/**
 * Returns a unary function which adds `a` to `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const add5 = BigInt.add(5n)
 *
 * add5(10n)                   // 15n
 * ```
 *
 * @param a - the first bigint
 * @returns {Unary.t<T, T>}
 * @group Operables
 * @since 4.0.0
 */
export declare function add(a: T): Fn.Unary<T, T>;
/**
 * Divides `a` by `b`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.div(4n, 2n)              // 2n
 * BigInt.div(12n, 3n)             // 4n
 * ```
 *
 * @param a first value (top of division)
 * @param b second value (bottom of division)
 * @returns the result
 * @group Operables
 * @since 4.0.0
 */
export declare function div(a: T, b: T): T;
/**
 * Returns a unary function which divides `a` by `b`.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.div(4n, 2n)              // 2n
 * BigInt.div(12n, 3n)             // 4n
 * ```
 *
 * @param a second value (bottom of division)
 * @returns the unary function
 * @group Operables
 * @since 4.0.0
 */
export declare function div(a: T): Fn.Unary<T, T>;
/**
 * Returns the remainder of the division of `a` by `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.mod(2n, 2n)             // 0n
 * BigInt.mod(3n, 2n)             // 1n
 * ```
 *
 * @param a the value
 * @param b the modulus
 * @returns the remainder
 * @group Operables
 * @since 4.0.0
 */
export declare function mod(a: T, b: T): T;
/**
 * Returns a unary function which returns the remainder of the division of `b` by `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const mod2 = BigInt.mod(2n)
 *
 * mod2(10n)                   // 0n
 * mod2(15n)                   // 1n
 * ```
 *
 * @param a the value
 * @returns the unary function
 * @group Operables
 * @since 4.0.0
 */
export declare function mod(a: T): Fn.Unary<T, T>;
/**
 * Multiplies `a` to `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.mul(5n, -2n)             // -10n
 * BigInt.mul(5n, 12n)             // 60n
 * ```
 *
 * @param a the first bigint
 * @param b the second bigint
 * @returns the result
 * @group Operables
 * @since 4.0.0
 */
export declare function mul(a: T, b: T): T;
/**
 * Returns a `Unary<T, T>` function which once called multiplies `b` to `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const mul5 = BigInt.mul(5n)
 *
 * mul5(10n)                   // 50n
 * ```
 *
 * @param a the first bigint
 * @returns the unary function
 * @group Operables
 * @since 4.0.0
 */
export declare function mul(a: T): Fn.Unary<T, T>;
/**
 * Elevates `a` by `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.pow(2n, 3n)             // 8n
 * BigInt.pow(3n, 2n)             // 9n
 * ```
 *
 * @param a the first bigint
 * @param b the second bigint
 * @returns the result
 * @group Operables
 * @since 4.0.0
 */
export declare function pow(a: T, b: T): T;
/**
 * Returns a `Unary<T, T>` function which once called elevates `b` by `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const pow5 = BigInt.pow(5n)
 *
 * pow5(10n)                   // 100_000n
 * ```
 *
 * @group Operables
 * @since 4.0.0
 */
export declare function pow(a: T): Fn.Unary<T, T>;
/**
 * Root of `a` under `b` if both specified, otherwise returns a `Unary<T, T>`
 * function which once called returns the root of `b` under `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.root(4n, 2n)             // 2n
 * BigInt.root(9n, 2n)             // 3n
 * ```
 *
 * @param a the bigint
 * @param b the root
 * @group Operables
 * @since 4.0.0
 */
export declare function root(a: T, b: T): T;
/**
 * Returns a `Unary<T, T>` function which once called returns the root of `b` under `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const root2 = BigInt.root(2n)
 *
 * root2(4n)                   // 2n
 * root2(9n)                   // 3n
 * ```
 *
 * @param a the root
 * @returns the unary function
 * @group Operables
 * @since 4.0.0
 */
export declare function root(a: T): Fn.Unary<T, T>;
/**
 * Subtracts `b` to `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.sub(5n, -2n)             // 7n
 * BigInt.sub(5n, 12n)             // -7n
 * ```
 *
 * @param a the first bigint
 * @param b the second bigint
 * @returns a - b
 * @group Operables
 * @since 4.0.0
 */
export declare function sub(a: T, b: T): T;
/**
 * Returns a `Unary<T, T>` function which once called subtracts `a` to `b`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const sub5 = BigInt.sub(5n)
 *
 * sub5(10n)                   // 5n
 * sub5(-2n)                   // -7n
 * ```
 *
 * @param a
 * @returns the unary function (b - a)
 * @group Operables
 * @since 4.0.0
 */
export declare function sub(a: T): Fn.Unary<T, T>;
/**
 * Compares two bigints `a` and `b` if `b`
 *
 * Great to sort a bigint array in ASC direction.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const collection = [10n, 5n, 6n, 4n, 12n, 22n, 3n];
 *
 * collection.sort(BigInt.asc)     // [3n, 4n, 5n, 6n, 10n, 12n, 22n]
 * ```
 *
 * @param a the first comparison bigint
 * @param b the second comparison bigint
 * @returns the result:
 *  - -1 if a is less than b
 *  - 0 if a is equal to b
 *  - 1 if a is greater than b
 * @group Sortables
 * @since 4.0.0
 */
export declare function asc(a: T, b: T): Functors.ComparableResult;
/**
 * Returns a `Unary<T, T>` function which once called compares `b` and `a`
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.asc(1n)(2n) // 1
 * BigInt.asc(1n)(1n) // 0
 * BigInt.asc(1n)(0n) // -1
 * ```
 *
 * @param a the bigint
 * @returns the comparator unary function
 * @group Sortables
 * @since 4.0.0
 */
export declare function asc(a: T): Fn.Unary<T, Functors.ComparableResult>;
/**
 * Compares two numbers `b` and `a` if `b` is defined, otherwise returns a
 * `Unary<number, number>` function which once called compares `a` and `b`
 *
 * Great to sort a numeric array in DESC direction.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * const collection = [10n, 5n, 6n, 4n, 12n, 22n, 3n];
 *
 * collection.sort(BigInt.desc)     // [22n, 12n, 10n, 6n, 5n, 4n, 3n]
 * ```
 *
 * @param a the first comparison bigint
 * @param b the second comparison bigint
 * @returns the result:
 *  - -1 if a is more than b
 *  - 0 if a is equal to b
 *  - 1 if a is less than b
 * @group Sortables
 * @since 4.0.0
 */
export declare function desc(a: T, b: T): Functors.ComparableResult;
export declare function desc(a: T): Fn.Unary<T, Functors.ComparableResult>;
/**
 * Returns a bigint in binary notation.
 *
 * If the passed argument at runtime is not a bigint, an Error will be returned.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.toBin(10n)        // "0b1010"
 * ```
 *
 * @param a the bigint
 * @returns the binary bigint value
 * @group Serializables
 * @since 4.0.0
 */
export declare const toBin: Fn.Unary<bigint, Result.T<string>>;
/**
 * Returns a bigint in hexadecimal notation
 *
 * If the passed argument at runtime is not a bigint, an Error will be returned.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.toHex(10n)        // "0xa"
 * ```
 *
 * @param a the bigint
 * @returns the hexadecimal bigint value
 * @group Serializables
 * @since 4.0.0
 */
export declare const toHex: Fn.Unary<bigint, Result.T<string>>;
/**
 * Returns a bigint in octal notation
 *
 * If the passed argument at runtime is not a bigint, an Error will be returned.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.toOct(10n)        // "0o12"
 * ```
 *
 * @param a the bigint
 * @returns the octal bigint value
 * @group Serializables
 * @since 4.0.0
 */
export declare const toOct: Fn.Unary<bigint, Result.T<string>>;
/**
 * Returns a bigint in json notation.
 *
 * If the passed argument at runtime is not a bigint, an Error will be returned.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.toJSON(10n)       // "10"
 * ```
 *
 * @param a the bigint
 * @returns the json bigint value
 * @group Serializables
 * @since 4.0.0
 */
export declare const toJSON: Fn.Unary<bigint, Result.T<string>>;
/**
 * Returns a stringified number.
 *
 * If the passed argument at runtime is not a bigint, an Error will be returned.
 *
 * @example
 *
 * ```ts
 * import { BigInt } from 'tiinvo';
 *
 * BigInt.toString(10n)       // "10"
 * ```
 *
 * @param a the bigint
 * @returns the string
 * @group Serializables
 * @since 4.0.0
 */
export declare const toString: Fn.Unary<bigint, Result.T<string>>;
