import { catchableAsync, catchableSync } from './Functors.js';
import { make as makecatch } from './Catch.js';
/**
 * Returns `true` if the option is `None`, `false` otherwise.
 *
 * ```typescript
 * import { Option } from 'tiinvo';
 *
 * Option.isNone(1);          // false
 * Option.isNone(null);       // true
 * Option.isNone(undefined);  // true
 * ```
 *
 * @param x  the argument to check if none
 * @returns
 *  - `true` if `x` is `None`
 *  - `false` otherwise
 * @since 4.0.0
 */
export const isNone = (x) => x === undefined || x === null;
/**
 * Returns `true` if the option is `Some<unknown>`, `false` otherwise.
 *
 * ```typescript
 * import { Option } from 'tiinvo';
 *
 * Option.isSome(1)         // true
 * Option.isSome(null)      // false
 * Option.isSome(undefined) // false
 * ```
 *
 * @param x
 * @returns
 *  - `true` if `x` is `Some<unknown>`
 *  - `false` otherwise
 * @since 4.0.0
 */
export const isSome = (x) => x !== undefined && x !== null;
/**
 * Returns `true` if the option is `Some<A>` and the value type is satisfied by the guard, otherwise `false`.
 *
 * If the option is `None`, it will always return `true`.
 *
 * ```typescript
 * import { Num, Option } from 'tiinvo';
 *
 * const x = 1
 * const y = null
 * const z = undefined
 * const w = `a`
 *
 * const isnumsome = Option.guardOf(Num.guard);
 *
 * isnumsome(x) // true
 * isnumsome(y) // true
 * isnumsome(z) // true
 * isnumsome(w) // false
 *
 * ```
 *
 * @template A the type
 * @param f the guard
 * @returns the new `Guardable<T<A>>` which returns
 *  - `true` if `x` is `None` or is `Some<A>`
 *  - `false` otherwise
 * @since 4.0.0
 */
export const guardOf = (f) => (x) => isNone(x) ? true : f(x);
export function cmp(c, a, b) {
    const _cmp = (x, y) => {
        if (isNone(x) && isNone(y)) {
            return 0;
        }
        if (isNone(x)) {
            return -1;
        }
        if (isNone(y)) {
            return 1;
        }
        return typeof c === 'function' ? c(x, y) : c.cmp(x, y);
    };
    switch (arguments.length) {
        case 3: return _cmp(a, b);
        case 2: return (b) => _cmp(b, a);
        case 1: return (a, b) => _cmp(a, b);
    }
}
;
export function eq(e, a, b) {
    const _eq = (x, y) => {
        if (isSome(x) && isSome(y)) {
            return typeof e === 'function' ? e(x, y) : e.eq(x, y);
        }
        else if (isNone(x) && isNone(y)) {
            return true;
        }
        return false;
    };
    switch (arguments.length) {
        case 3: return _eq(a, b);
        case 2: return (b) => _eq(b, a);
        case 1: return (a, b) => _eq(a, b);
    }
}
;
export function filter(a, b) {
    if (arguments.length === 2) {
        return isNone(b) ? null : a(b) ? b : null;
    }
    return (c) => isNone(c) ? null : a(c) ? c : null;
}
export function map(m, a) {
    const _map = (x) => isNone(x) ? null : m(x);
    if (typeof m === 'function' && arguments.length === 2) {
        return _map(a);
    }
    return _map;
}
;
export function mapOr(m, a, b) {
    const _mor = (x, or) => isNone(x) ? or : m(x);
    if (arguments.length === 3) {
        return _mor(a, b);
    }
    return (c) => _mor(c, a);
}
//#endregion
//#region catchable
/**
 * Calls a function `F` with it's arguments and returns a `Promise<Option.T<ReturnType<f>>>`
 *
 * ```typescript
 * import { Option, Num } from 'tiinvo';
 *
 * const fn = async (arg: number) => {
 *   if (Num.isEven(arg)) {
 *     return arg * 2;
 *   }
 *
 *   throw new Error(`${arg} is not even`);
 * }
 *
 * const safe = Option.tryAsync(fn);
 *
 * await safe(2) // 4
 * await safe(3) // null
 *
 * Option.isSome(await safe(2)) // true
 * Option.isNone(await safe(3)) // true
 * ```
 *
 * @template F the function type
 * @param f the function to wrap
 * @returns the wrapped function
 *  - returns `Promise<None>` if the function catched
 *  - otherwise the function returns `Promise<ReturnValue<F>>`
 * @since 4.0.0
 */
export const tryAsync = (f) => {
    return makecatch({
        [catchableAsync]() {
            return {
                catch: async (_err, _args) => null,
                func: f
            };
        }
    });
};
/**
 * Calls a function `F` with it's arguments and returns a `Option.T<ReturnType<f>>`
 *
 * ```typescript
 * import { Option, Num } from 'tiinvo';
 *
 * const fn = (arg: number) => {
 *   if (Num.isEven(arg)) {
 *     return arg * 2;
 *   }
 *
 *   throw new Error(`${arg} is not even`);
 * }
 *
 * const safe = Option.trySync(fn);
 *
 * safe(2) // 4
 * safe(3) // null
 *
 * Option.isSome(safe(2)) // true
 * Option.isNone(safe(3)) // true
 * ```
 *
 * @template F the function type
 * @param f the function to wrap
 * @returns the wrapped function
 *  - returns `None` if the function catched
 *  - otherwise the function returns `ReturnValue<F>`
 * @since 4.0.0
 */
export const trySync = (f) => {
    return makecatch({
        [catchableSync]() {
            return {
                catch: () => null,
                func: f
            };
        }
    });
};
//#endregion
