import type * as Functors from './Functors.js';
import type * as Fn from './Fn.js';
/**
 * Represents a numeric range from a starting value `start` to an ending value `end`.
 */
export type T = {
    readonly start: number;
    readonly end: number;
    readonly step: number;
    [Symbol.iterator](): Iterator<number>;
};
/**
 * Makes a new range from a start to an end (inclusive)
 *
 * If the third parameter `step` is passed, the range will increment by that step
 *
 * **Important**: the range accepts negative values, and the `end` does not have to be greater than the `start`.
 *
 * @example
 *
 * ```ts
 * import { Range } from 'tiinvo'
 *
 * for (const n of Range.make(0, 10)) {
 *    console.log(n)
 * }
 *
 * // will log 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10
 *
 * for (const n of Range.make(0, 10, 2)) {
 *    console.log(n)
 * }
 *
 * // will log 0, 2, 4, 6, 8, 10
 *
 * for (const n of Range.make(10, 0, 2)) {
 *    console.log(n)
 * }
 *
 * // will log 10, 8, 6, 4, 2, 0
 * ```
 *
 * @param start range's starting value
 * @param end range's ending value
 * @param step range's increment value
 * @group Factories
 * @since 4.0.0
 */
export declare const make: (start: number, end: number, step?: number) => T;
/**
 * Checks if a parameter `x` is of `Range.T` type
 *
 * @example
 *
 * ```ts
 * import { Range } from 'tiinvo'
 *
 * Range.guard(10)                                // false
 * Range.guard([])                                // false
 * Range.guard({})                                // false
 * Range.guard({ start: 10, end: 20 })            // false
 * Range.guard({ start: 10, end: 20, step: 12 })  // false
 * Range.guard(Range.make(1, 2))                  // true
 * ```
 *
 * @param x the value to check
 * @returns true if `x` is a `T`, false otherwise
 * @group Guardables
 * @since 4.0.0
 */
export declare const guard: (x: unknown) => x is T;
/**
 * Checks whenever a number is within in a `Range.T`
 *
 * @example
 *
 * ```ts
 * import { Range } from 'tiinvo'
 *
 * const r = Range.make(3, 8)
 *
 * Range.inRange(r, 10)   // false
 * Range.inRange(r, 6)    // true
 * ```
 *
 * @param t the range
 * @param a the value to check
 * @group Predicates
 * @since 4.0.0
 */
export declare function inRange(t: T, a: number): boolean;
/**
 * Returns a unary function which checks whenever a `Range.T` contains a value `t`
 *
 * @example
 *
 * ```ts
 * import { Range } from 'tiinvo'
 *
 * const r = Range.make(3, 8)
 *
 * Range.inRange(10)(r)   // false
 * Range.inRange(6)(r)    // true
 * ```
 *
 * @param t the value to check
 * @returns the unary function `Fn.Unary<T, boolean>`
 * @group Predicates
 * @since 4.0.0
 */
export declare function inRange(t: number): Fn.Unary<T, boolean>;
/**
 * Maps a functor `Mappable<number, A>` over a `Range.T`
 *
 * @example
 *
 * ```ts
 * import { Range, Num } from 'tiinvo'
 *
 * const r = Range.make(20, 30)
 *
 * Range.map(r, Num.toHex)   // ['0x14', '0x15', '0x16', '0x17', '0x18', '0x19', '0x1a', '0x1b', '0x1c', '0x1d', '0x1e']
 * ```
 *
 * @template A the mapped value type
 * @param t the Range
 * @param m the Mappable functor
 * @returns A[] if `t` is `Range.T`
 * @group Mappables
 * @since 4.0.0
 */
export declare function map<A>(t: T, m: Functors.Mappable<number, A>): A[];
/**
 * Returns a unary function which maps a functor `Mappable<number, A>` over a `Range.T`
 *
 * @example
 *
 * ```ts
 * import { Range, Num } from 'tiinvo'
 *
 * const r = Range.make(20, 30)
 * const m = Range.map(Num.toHex)
 *
 * m(r)   // ['0x14', '0x15', '0x16', '0x17', '0x18', '0x19', '0x1a', '0x1b', '0x1c', '0x1d', '0x1e']
 * ```
 *
 * @template A the mapped value type
 * @param t the Range
 * @returns the unary function
 * @group Mappables
 * @since 4.0.0
 */
export declare function map<A>(t: Functors.Mappable<number, A>): Fn.Unary<T, A[]>;
/**
 * Converts a `Range.T` to a `number[]` array
 *
 * @example
 *
 * ```ts
 * import { Range } from 'tiinvo'
 *
 * const r = Range.make(0, 10)
 *
 * Range.toArray(r)   // [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
 * ```
 *
 * @group Serializables
 * @since 4.0.0
 */
export declare const toArray: {
    <T_1>(arrayLike: ArrayLike<T_1>): T_1[];
    <T_2, U>(arrayLike: ArrayLike<T_2>, mapfn: (v: T_2, k: number) => U, thisArg?: any): U[];
    <T_3>(iterable: Iterable<T_3> | ArrayLike<T_3>): T_3[];
    <T_4, U_1>(iterable: Iterable<T_4> | ArrayLike<T_4>, mapfn: (v: T_4, k: number) => U_1, thisArg?: any): U_1[];
};
