import { cmp as arrCmp, eq as arrEq } from './Arr.js';
import { isErr } from './Result.js';
const indexer = Symbol("indexer");
//#region factories
/**
 * Try to deserialize a `string` to a `Sequence.T<A>`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const x  = Sequence.fromString<number>(`10,20,30`) as Sequence.T<number>;
 * const t1 = Sequence.make(10, 20, 30)
 * const t2 = Sequence.make(10, 30, 20)
 *
 * Sequence.eq(x, t1)        // true
 * Sequence.eq(x, t2)        // false
 * ```
 *
 * @template A the Sequence's expected element type
 * @param x the string
 * @returns
 *  - `Result.Ok<Sequence.T<A>>` if the deserialization is successful
 *  - `Result.Err` otherwise
 * @group Factories
 * @since 4.0.0
 */
export const fromString = (x) => {
    try {
        return make(JSON.parse(`[${x}]`));
    }
    catch (err) {
        return err;
    }
};
export function make(...v) {
    const values = {};
    if (arguments.length === 1 && Array.isArray(arguments[0])) {
        v = arguments[0];
    }
    for (let i = 0; i < v.length; i++) {
        values[i] = v[i];
    }
    const _t = {
        [indexer]() {
            return Object.freeze({ ...values });
        },
    };
    Object.defineProperty(_t, Symbol.iterator, {
        enumerable: false,
        writable: false,
        configurable: true,
        value: function () {
            let i = 0;
            let keys = Object.keys(values);
            return {
                next: () => {
                    return {
                        value: values[keys[i++]],
                        done: (i > keys.length)
                    };
                }
            };
        }
    });
    return _t;
}
;
//#endregion
//#region guardables
/**
 * Checks if a value `x` is a `Sequence.T<unknown>`.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * Sequence.guard(10)                 // false
 * Sequence.guard([10, 20, 30])       // false
 * Sequence.guard(Sequence.make())    // true
 * ```
 *
 * @param x the value to guard
 * @returns
 *  - `true` if `x` is `Sequence.T<unknown>`
 *  - `false` otherwise
 * @group Guardables
 * @since 4.0.0
 */
export const guard = (x) => typeof x === 'object' && x !== null && indexer in x;
/**
 * Checks if the parameter `x` is a `Sequence.T<A>`
 *
 * @example
 *
 * ```ts
 * import { Sequence, Num, Str } from 'tiinvo'
 *
 * const s0 = Sequence.make<number | string>(1, 'hello', 2)
 * const s1 = Sequence.make('hello', 'world')
 * const isStrSequence = Sequence.guardOf(Str.guard);
 *
 * isStrSequence(s0)      // false
 * isStrSequence(s1)      // true
 * ```
 *
 * @template a the type
 * @param g the guard
 * @returns the new guard which returns
 *  - `true` if `x` is a `Sequence.T<A>`
 *  - `false` otherwise
 * @group Guardables
 * @since 4.0.0
 */
export const guardOf = (g) => (x) => guard(x) && toArray(x).every(g);
export function cmp(mod, a, b) {
    const c = arrCmp(typeof mod === 'function' ? mod : mod.cmp);
    if (guard(a) && guard(b)) {
        return c(toArray(a), toArray(b));
    }
    else if (guard(a) && !guard(b)) {
        return (d) => c(toArray(d), toArray(a));
    }
    else {
        return (d, e) => c(toArray(d), toArray(e));
    }
}
export function eq(mod, a, b) {
    const eqfn = arrEq(typeof mod === 'function' ? mod : mod.eq);
    if (guard(a) && guard(b)) {
        return eqfn(toArray(a), toArray(b));
    }
    else if (guard(a) && !guard(b)) {
        return (d) => eqfn(toArray(a), toArray(d));
    }
    else {
        return (d, e) => eqfn(toArray(d), toArray(e));
    }
}
export function map(a, m) {
    if (guard(a) && typeof m === 'function') {
        return make(toArray(a).map(m));
    }
    return (b) => {
        return make(toArray(b).map(a));
    };
}
export function reduce(a, mod, s) {
    if (guard(a) && typeof mod === 'function') {
        return Array.from(a).reduce((x, c) => mod(x, c), s);
    }
    return (b) => Array.from(b).reduce((x, c) => a(x, c), mod);
}
export function filter(a, f) {
    if (guard(a) && typeof f === 'function') {
        return make(Array.from(a).filter(f));
    }
    return (c) => make(Array.from(c).filter(a));
}
export function filterReduce(a, mod) {
    const impl = (x, y) => {
        let out = y.default;
        for (const e of x) {
            if (y.filter(e)) {
                out = y.reduce(out, e);
            }
        }
        return out;
    };
    if (guard(a)) {
        return impl(a, mod);
    }
    return (b) => impl(b, a);
}
export function filterMap(a, f) {
    const handle = (a, mod) => {
        const outvalues = [];
        for (const v of a) {
            if (mod.filter(v)) {
                outvalues.push(mod.map(v));
            }
        }
        return make(outvalues);
    };
    if (guard(a) && typeof f === 'object') {
        return handle(a, f);
    }
    return (b) => handle(b, a);
}
export function append(a, b) {
    if (guard(a)) {
        return make(Object.values(a[indexer]()).concat(b));
    }
    return (b) => make(Object.values(b[indexer]()).concat(a));
}
export function concat(a, b) {
    if (guard(a) && guard(b)) {
        return make([...a, ...b]);
    }
    return (b) => make([...b, ...a]);
}
export function prepend(a, b) {
    if (guard(a)) {
        return make([b].concat([...a]));
    }
    return (b) => make([a].concat([...b]));
}
export function count(a, p) {
    if (guard(a) && typeof p === 'function') {
        return length(filter(a, p));
    }
    return (b) => length(filter(b, a));
}
/**
 * Gets a `Sequence.T<A>`'s first element.
 *
 * Returns `Option.None` if none is found.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s0 = Sequence.make(10, 20, 30)
 * const s1 = Sequence.make()
 *
 * Sequence.first(s0)       // 10
 * Sequence.first(s1)       // null
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the first element of the sequence:
 *  - `Option.Some<A>` if the sequence has at least one element
 *  - `Option.None` otherwise
 * @group Accessors
 * @since 4.0.0
 */
export const first = (t) => {
    const f = get(t, 0);
    return isErr(f) ? null : f;
};
export function get(a, i) {
    const _get = (x, y) => {
        const s = length(x);
        if (y < 0 || y > s - 1) {
            return new RangeError(`Index out of bounds ${y} for length ${s}`);
        }
        return values(x)[y];
    };
    if (guard(a)) {
        return _get(a, i);
    }
    return (b) => _get(b, a);
}
/**
 * Gets a `Sequence.T<A>`'s last element if any.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s0 = Sequence.make(10, 20, 30)
 * const s1 = Sequence.make()
 *
 * Sequence.last(s0)       // 30
 * Sequence.last(s1)       // null
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns
 *  - `Option.Some<A>` if the sequence is not empty
 *  - `Option.None` otherwise
 * @group Accessors
 * @since 4.0.0
 */
export const last = (t) => {
    const r = get(t, length(t) - 1);
    return isErr(r) ? null : r;
};
/**
 * Gets the length of a `Sequence.T<A>`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make(1, 2, 3)
 *
 * Sequence.length(s)           // 3
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the Sequence's length
 * @group Accessors
 * @since 4.0.0
 */
export const length = (t) => Object.values(t[indexer]()).length;
/**
 * Gets values of a `Sequence.T<A>` as an immutable indexed object.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make('hello', 'world')
 *
 * Sequence.values(s)       // { 0: 'hello', 1: 'world' }
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the sequence values as an immutable dictionary
 * @group Accessors
 * @since 4.0.0
 */
export const values = (t) => t[indexer]();
//#endregion
//#region predicates
/**
 * Returns `true` if the sequence is empty.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make()
 * const s1 = Sequence.make(10)
 *
 * Sequence.empty(s)                // true
 * Sequence.empty(s1)               // false
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns
 *  - 'true' if the sequence is empty
 *  - 'false' otherwise
 * @group Predicates
 * @since 4.0.0
 */
export const empty = (t) => length(t) === 0;
/**
 * Returns `true` if the sequence is populated.
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make(10, 20, 30)
 *
 * Sequence.populated(s)                // true
 * Sequence.populated(Sequence.make())  // false
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns
 *  - 'true' if the sequence is populated
 *  - 'false' otherwise
 * @group Predicates
 * @since 4.0.0
 */
export const populated = (t) => length(t) !== 0;
export function sort(a, mod) {
    const getcmp = (x) => typeof x === 'function' ? x : x.cmp;
    if (arguments.length === 2 && guard(a)) {
        return make(Array.from(a).sort(getcmp(mod)));
    }
    return (x) => make(Array.from(x).sort(getcmp(a)));
}
//#endregion
//#region serializables
/**
 * Converts a `Sequence.T<A>` to an array `a[]`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make(10, 20, 30)
 *
 * Sequence.toArray(s) // [10, 20, 30]
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the array
 * @group Serializables
 * @since 4.0.0
 */
export const toArray = (t) => Array.from(t);
/**
 * Serializes a `Sequence.T<A>` to json
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * Sequence.toJSON(Sequence.make(1, 2))     // { 0: 1, 1: 2 }
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the JSONised value
 * @group Serializables
 * @since 4.0.0
 */
export const toJSON = (t) => Array.from(t);
/**
 * Serializes a `Sequence.T<A>` to a `Map<number, a>`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * Sequence.toMap(Sequence.make(1, 2))     // Map([0, 1], [1, 2])
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the Map
 * @group Serializables
 * @since 4.0.0
 * @since 4.0.0
 */
export const toMap = (t) => new Map(Object.entries(t[indexer]()));
/**
 * Converts a `Sequence.T<A>` to a Set `Set<a>`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make(10, 20, 30)
 *
 * Sequence.toSet(s) // Set(10, 20, 30)
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the Set
 * @group Serializables
 * @since 4.0.0
 */
export const toSet = (t) => new Set(t);
/**
 * Stringifies a `Sequence.T<A>`
 *
 * @example
 *
 * ```ts
 * import { Sequence } from 'tiinvo'
 *
 * const s = Sequence.make(10, 20, 30)
 *
 * Sequence.toString(s)     // '10,20,30'
 * ```
 *
 * @template A the Sequence's element type
 * @param t the sequence
 * @returns the string
 * @group Serializables
 * @since 4.0.0
 */
export const toString = (t) => Array.from(t).toString();
//#endregion
