//#region guards
/**
 * Checks if a value `x` is a string `T`
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.guard(10)            // false
 * Str.guard({})            // false
 * Str.guard([])            // false
 * Str.guard(null)          // false
 * Str.guard("hello world") // true
 * ```
 *
 * @param x the value to check
 * @returns
 *  - true if `x` is a string `T`
 *  - false otherwise
 * @group Guardables
 * @since 4.0.0
 */
export const guard = (x) => typeof x === 'string';
export function cmp(a, b) {
    const _cmp = (x, y) => x > y ? 1 : x < y ? -1 : 0;
    if (guard(a) && guard(b)) {
        return _cmp(a, b);
    }
    return (b) => _cmp(b, a);
}
;
export function eq(a, b) {
    if (guard(a) && guard(b)) {
        return a === b;
    }
    return (b) => b === a;
}
export function asc(a, b) {
    if (guard(b)) {
        return cmp(a, b);
    }
    return (b) => cmp(b, a);
}
export function desc(a, b) {
    if (guard(b)) {
        return cmp(b, a);
    }
    return (b) => cmp(a, b);
}
//#endregion
//#region native methods
/**
 * Formats a string to camel case.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.camel('hello world'); // 'helloWorld'
 * ```
 *
 * @param a the string
 * @returns the camelCased string
 * @group Natives
 * @since 4.0.0
 */
export const camel = a => a.replace(/\s(.)/g, (_, c) => c.toUpperCase());
export function charAt(a, b) {
    if (guard(a) && typeof b === 'number') {
        const c = a.charAt(b);
        return c === "" ? null : c;
    }
    else if (typeof a !== 'number') {
        return (_) => null;
    }
    return (b) => {
        const c = b.charAt(a);
        return c === "" ? null : c;
    };
}
export function charCodeAt(a, b) {
    if (guard(a) && typeof b === 'number') {
        const c = a.charCodeAt(b);
        return c >= 0 ? c : null;
    }
    else if (typeof a !== 'number') {
        return (_) => null;
    }
    return (b) => {
        const c = b.charCodeAt(a);
        return c >= 0 ? c : null;
    };
}
/**
 * Returns an array of characters.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.chars('hello'); // ['h','e','l','l','o']
 * ```
 *
 * @param a the string
 * @returns the array of characters
 * @group Natives
 * @since 4.0.0
 */
export const chars = a => a.split('');
export function concat(a, b) {
    if (guard(a) && guard(b)) {
        return a.concat(b);
    }
    return (b) => b.concat(a);
}
export function endsWith(a, b) {
    if (guard(a) && guard(b)) {
        return a.endsWith(b);
    }
    return (b) => b.endsWith(a);
}
export function includes(a, b) {
    if (guard(a) && guard(b)) {
        return a.includes(b);
    }
    return (b) => b.includes(a);
}
export function indexOf(a, b, i) {
    if (guard(a) && guard(b)) {
        return a.indexOf(b, i);
    }
    return (b, i) => b.indexOf(a, i);
}
export function lastIndexOf(a, b, p) {
    if (guard(a) && guard(b)) {
        return a.lastIndexOf(b, p);
    }
    return (b, p) => b.lastIndexOf(a, p);
}
/**
 * Returns a string length
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.length('hello'); // 5
 * ```
 *
 * @param a the string
 * @returns the length of the string `a`
 * @group Natives
 * @since 4.0.0
 */
export const length = a => a.length;
/**
 * Returns an array of lines in a string.
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.lines('hello\nworld'); // ['hello', 'world']
 * ```
 *
 * @param x the string
 * @returns the array containing the lines
 * @group Natives
 * @since 4.0.0
 *
 */
export const lines = x => x.split(/\r?\n/);
/**
 * Returns a lowercased string
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.lower('HELLO'); // "hello"
 * ```
 *
 * @param a the string
 * @returns the lowercased string
 * @group Natives
 * @since 4.0.0
 */
export const lower = a => a.toLowerCase();
export function match(a, b) {
    if (guard(a) && (guard(b) || b instanceof RegExp)) {
        return a.match(b);
    }
    return (b) => b.match(a);
}
export function padEnd(a, b, c) {
    if (guard(a) && typeof b === 'number') {
        return a.padEnd(b, c);
    }
    return (x, y) => x.padEnd(a, b ?? y);
}
/**
 * Formats a string to pascal case.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.pascal('hello world'); // 'HelloWorld'
 * ```
 *
 * @param a the string
 * @returns the PascalCased string
 * @group Natives
 * @since 4.0.0
 */
export const pascal = a => a.replace(/\s(.)/g, (_, c) => c.toUpperCase()).replace(/^(.)/, (_, c) => c.toUpperCase());
export function padStart(a, b, c) {
    if (guard(a) && typeof b === 'number') {
        return a.padStart(b, c);
    }
    return (x, y) => x.padStart(a, y ?? b);
}
export function repeat(a, b) {
    if (guard(a) && typeof b === 'number') {
        return a.repeat(b);
    }
    return (b) => b.repeat(a);
}
export function replace(a, b, c) {
    if (guard(a) && guard(b) && typeof c !== 'undefined') {
        return a.replace(b, c);
    }
    return (x) => x.replace(a, b);
}
/**
 * Reverses a string.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.reverse('hello'); // 'olleh'
 * ```
 *
 * @param a the string to reverse
 * @returns the reversed string
 * @group Natives
 * @since 4.0.0
 */
export const reverse = a => a.split('').reverse().join('');
export function search(a, b) {
    const m = x => x >= 0 ? x : null;
    if (guard(a) && typeof b !== 'undefined') {
        return m(a.search(b));
    }
    return (b) => m(b.search(a));
}
export function slice(a, b, c) {
    if (guard(a) && typeof b === 'number') {
        return a.slice(b, c);
    }
    return (x, y) => x.slice(a, b ?? y);
}
export function split(a, b, c) {
    if (guard(a) && (guard(b) || b instanceof RegExp || (!!b && typeof b === 'object' && Symbol.split in b))) {
        return a.split(b, c);
    }
    return (x, y) => x.split(a, b ?? y);
}
/**
 * Trims a string both at start an the end of it.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.trim('    hello world    '); // 'hello world'
 * ```
 *
 * @param a the string
 * @returns the trimmed string
 * @group Natives
 * @since 4.0.0
 */
export const trim = a => a.trim();
/**
 * Trims a string at it's end.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.trimEnd('    hello world    '); // '    hello world'
 * ```
 *
 * @param a the string
 * @returns the trimmed string
 * @group Natives
 * @since 4.0.0
 */
export const trimEnd = a => a.trimEnd();
/**
 * Trims a string at it's start.
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.trimStart('    hello world    '); // 'hello world    '
 * ```
 *
 * @param a the string
 * @returns the trimmed string
 * @group Natives
 * @since 4.0.0
 */
export const trimStart = a => a.trimStart();
/**
 * Returns a uppercased string
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.upper('hello'); // "HELLO"
 * ```
 *
 * @param a the string
 * @returns the uppercased string
 * @group Natives
 * @since 4.0.0
 */
export const upper = a => a.toUpperCase();
/**
 * Returns an array of words in a string.
 *
 * ```ts
 * import { Str } from 'tiinvo';
 *
 * Str.words('hello world'); // ['hello', 'world']
 * ```
 *
 * @param a the string
 * @returns the array containing the words in string `a`
 * @group Natives
 * @since 4.0.0
 */
export const words = a => a.split(/\s+/m);
//#endregion
//#region serializables
/**
 * Returns an array of chars
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toArray("hello")   // ["h", "e", "l", "l", "o"]
 * ```
 *
 * @param t the string
 * @returs the resulting char array
 * @group Serializables
 * @since 4.0.0
 */
export const toArray = (t) => t.split('');
/**
 * Returns an array of binarized char codes
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toBinArray("hello")   // ["0b1101000", "0b1100101", "0b1101100", "0b1101100", "0b1101111"]
 * ```
 *
 * @param t the string
 * @returs the resulting binary strings array
 * @group Serializables
 * @since 4.0.0
 */
export const toBinArray = (t) => toArray(t).map(x => `0b${x.charCodeAt(0).toString(2)}`);
/**
 * Returns an array of char codes
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toCharCodeArray("hello")   // [104, 101, 108, 108, 111]
 * ```
 *
 * @param t the string
 * @returs the resulting char code array
 * @group Serializables
 * @since 4.0.0
 */
export const toCharCodeArray = (t) => toArray(t).map(charCodeAt(0));
/**
 * Returns an array of hexadecimals string
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toHexArray("hello") // ["0x68", "0x65", "0x6c", "0x6c", "0x6f"]
 * ```
 *
 * @param t the string
 * @returs the resulting hex char code array
 * @group Serializables
 * @since 4.0.0
 */
export const toHexArray = (t) => toArray(t).map(x => '0x' + charCodeAt(x, 0).toString(16));
/**
 * Serializes a string `T` to an `Int32Array`
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toInt32Array('hello') // Int32Array(5) [ 104, 101, 108, 108, 111 ]
 * ```
 *
 * @group
 * @since
 */
export const toInt32Array = (t) => Int32Array.from(t.split('').map(x => x.charCodeAt(0)));
/**
 * Returns an array of octal strings
 *
 * @example
 *
 * ```ts
 * import { Str } from 'tiinvo'
 *
 * Str.toHexArray("hello") // ["0o150", "0o145", "0o154", "0o154", "0o157"]
 * ```
 *
 * @param t the string
 * @returs the resulting oct char code array
 * @group Serializables
 * @since 4.0.0
 */
export const toOctArray = (t) => toArray(t).map(x => '0o' + charCodeAt(x, 0).toString(8));
//#endregion
