import { cmp as arrcmp } from './Arr.js';
const guardsymbolKey = Symbol('guard');
const guardsymbolValue = Symbol('guard');
const valuessymbol = Symbol('values');
//#region factories
/**
 * Creates a new `TypedMap.T<K, V>`.
 *
 * @example
 *
 * ```ts
 * import { Str, Num, TypedMap } from 'tiinvo'
 *
 * const m = new Map<string, number>([['hello', 100], ['world', 200]])
 *
 * TypedMap.make(Str, Num)
 * TypedMap.make(Str, Num, m)
 * TypedMap.make(Str.guard, Num)
 * TypedMap.make(Str.guard, Num, m)
 * TypedMap.make(Str, Num.guard)
 * TypedMap.make(Str, Num.guard, m)
 * TypedMap.make(Str.guard, Num.guard)
 * TypedMap.make(Str.guard, Num.guard, m)
 * ```
 *
 * @template K the key type
 * @template V the value type
 * @returns the `TypedMap.T<K, V>`
 * @group Factories
 * @since 4.0.0
 */
export function make(kg, vg, entries = []) {
    const _kg = typeof kg === 'function' ? kg : kg.guard;
    const _vg = typeof vg === 'function' ? vg : vg.guard;
    for (const [k, v] of entries) {
        if (!_kg(k)) {
            throw new TypeError(`Key ${k} is not of a valid type`);
        }
        if (!_vg(v)) {
            throw new TypeError(`Value ${v} is not of a valid type`);
        }
    }
    const map = new Map(entries);
    const _t = {
        get [guardsymbolKey]() {
            return typeof kg === 'function' ? kg : kg.guard;
        },
        get [guardsymbolValue]() {
            return typeof vg === 'function' ? vg : vg.guard;
        },
        get [valuessymbol]() {
            return new Map(map);
        },
    };
    Object.defineProperty(_t, Symbol.iterator, {
        enumerable: false,
        writable: false,
        configurable: true,
        value: function () {
            let i = 0;
            let keys = Array.from(map.keys());
            return {
                next: () => {
                    return {
                        value: map.get(keys[i++]),
                        done: (i > keys.length)
                    };
                }
            };
        }
    });
    return _t;
}
//#endregion
//#region guards
/**
 * Checks if `x` is a `TypedMap.T<unknown, unknown>`
 *
 * @example
 *
 * ```ts
 * import { TypedMap, Num, Str } from 'tiinvo'
 *
 * TypedMap.guard(10)                           // false
 * TypedMap.guard({})                           // false
 * TypedMap.guard(TypedMap.make(Num, Str))   // true
 * ```
 *
 * @param x the value to check
 * @returns
 *  - `true` if `x` is a `TypedMap.T<unknown, unknown>`
 *  - `false` otherwise
 * @group Guardables
 * @since 4.0.0
 */
export const guard = (x) => typeof x === 'object' && x !== null && guardsymbolKey in x && guardsymbolValue in x && valuessymbol in x;
export function guardOf(kg, vg, x) {
    const _kg = typeof kg === 'function' ? kg : kg.guard;
    const _vg = typeof vg === 'function' ? vg : vg.guard;
    const _g = (_x) => {
        if (guard(_x)) {
            for (const [k, v] of _x[valuessymbol]) {
                if (!_kg(k)) {
                    return false;
                }
                if (!_vg(v)) {
                    return false;
                }
            }
            return true;
        }
        return false;
    };
    if (arguments.length === 3) {
        return _g(x);
    }
    else {
        return (y) => _g(y);
    }
}
export function cmp(kmod, vmod, a, b) {
    const _kmod = typeof kmod === 'function' ? kmod : kmod.cmp;
    const _vmod = typeof vmod === 'function' ? vmod : vmod.cmp;
    const _cmp = (_kmod, _vmod, _a, _b) => {
        const keysresult = arrcmp(_kmod, Array.from(keys(_a)), Array.from(keys(_b)));
        const valuessresult = arrcmp(_vmod, Array.from(values(_a)), Array.from(values(_b)));
        return keysresult !== 0 ? keysresult : valuessresult;
    };
    if (arguments.length === 4 && guard(a) && guard(b)) {
        return _cmp(_kmod, _vmod, a, b);
    }
    else if (arguments.length === 3 && guard(a)) {
        return (c) => _cmp(_kmod, _vmod, c, a);
    }
    else {
        return (c, d) => _cmp(_kmod, _vmod, c, d);
    }
}
export function eq(kmod, vmod, a, b) {
    if (arguments.length === 4 && guard(a) && guard(b)) {
        return cmp(kmod, vmod, a, b) === 0;
    }
    else if (arguments.length === 3 && guard(a)) {
        return (c) => cmp(kmod, vmod, a, c) === 0;
    }
    else {
        return (c, d) => cmp(kmod, vmod, c, d) === 0;
    }
}
//#endregion
//#region accessors
/**
 * Returns an iterable of key, value pairs for every entry in the map.
 *
 * @example
 *
 * ```ts
 * import { TypedMap, Str, Num } from 'tiinvo'
 *
 * const m = TypedMap.make(Str, Num, [['a', 100], ['b', 200]])
 *
 * for (const [k, v] of TypedMap.entries(m)) {
 *    console.log(k, v)
 * }
 * // logs
 * // 'a' 100
 * // 'b' 200
 * ```
 *
 * @template K the key type
 * @template V the value type
 * @param t the TypedMap
 * @returns t entries
 * @group Accessors
 * @since 4.0.0
 */
export const entries = (t) => t[valuessymbol].entries();
export function get(key, t) {
    if (arguments.length === 2 && guard(t)) {
        return t?.[valuessymbol].get(key) ?? null;
    }
    return (t) => t[valuessymbol].get(key) ?? null;
}
export function has(key, t) {
    const _has = (_x, _k) => {
        if (_x[guardsymbolKey](_k)) {
            return _x[valuessymbol].has(_k);
        }
        throw new TypeError("Invalid key type for TypedMap");
    };
    if (arguments.length === 2 && guard(t)) {
        return _has(t, key);
    }
    return (_t) => _has(_t, key);
}
/**
 * Returns an iterable of keys in the map
 *
 * @example
 *
 * ```ts
 * import { TypedMap, Str, Num } from 'tiinvo'
 *
 * const m = TypedMap.make(Str, Num, [["a", 1], ["b", 2]])
 *
 * Array.from(TypedMap.keys(m))   // ["a", "b"]
 * ```
 *
 * @template K the key type
 * @template V the value type
 * @param t the TypedMap
 * @returns the keys iterable
 * @group Accessors
 * @since 4.0.0
 */
export const keys = (t) => t[valuessymbol].keys();
/**
 * Returns the number of elements in the Map
 *
 * @example
 *
 * ```ts
 * import { TypedMap, Str, Num } from 'tiinvo'
 *
 * const m = TypedMap.make(Str, Num, [["a", 1], ["b", 2]])
 *
 * TypedMap.size(m)   // 2
 * ```
 *
 * @template K the key type
 * @template V the value type
 * @param t the TypedMap
 * @returns the TypedMap's size
 * @group Accessors
 * @since 4.0.0
 */
export const size = (t) => t[valuessymbol].size;
/**
 * Returns an iterable of values in the map
 *
 * @example
 *
 * ```ts
 * import { TypedMap, Str, Num } from 'tiinvo'
 *
 * const m = TypedMap.make(Str, Num, [["a", 1], ["b", 2]])
 *
 * Array.from(TypedMap.values(m))   // [1, 2]
 * ```
 *
 * @template K the key type
 * @template V the value type
 * @param t the TypedMap
 * @returns the keys iterable
 * @group Accessors
 * @since 4.0.0
 */
export const values = (t) => t[valuessymbol].values();
function _delete(key, t) {
    const __delete = (_k, _t) => {
        const _map = _t[valuessymbol];
        if (!_t[guardsymbolKey](_k)) {
            throw new TypeError("Invalid key type for TypedMap");
        }
        if (_map.has(_k)) {
            _map.delete(_k);
        }
        return make(_t[guardsymbolKey], _t[guardsymbolValue], _map);
    };
    if (arguments.length === 2 && guard(t)) {
        return __delete(key, t);
    }
    return (_t) => __delete(key, _t);
}
export { _delete as delete };
export function set(a, b, c) {
    const _set = (_k, _v, _t) => {
        const _m = new Map(_t[valuessymbol]);
        _m.set(_k, _v);
        return make(_t[guardsymbolKey], _t[guardsymbolValue], _m);
    };
    if (arguments.length === 3 && guard(c)) {
        return _set(a, b, c);
    }
    else if (arguments.length === 2 && !guard(b)) {
        return (d) => _set(a, b, d);
    }
    else if (guard(a)) {
        return (d, e) => _set(d, e, a);
    }
}
//#endregion
