import type * as Fn from './Fn.js';
import type * as Functors from './Functors.js';
import * as Sequence from './Sequence.js';
declare const guardsymbol: unique symbol;
/**
 * The typed version of a `Sequence.t<A>`.
 */
export type T<A> = Sequence.T<A> & {
    [guardsymbol]: Functors.Guardable<A>;
};
/**
 * Creates a `TypedSequence.t<A>` starting from a `Guardable<A>` functor
 *
 * You can add initial values as arguments.
 *
 * If an argument does pass the guard check, the function will throw a TypeError
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * for (const x of TypedSequence.make(Num.guard, 10, 20, 30)) {
 *    console.log(x)
 * }
 * // 10
 * // 20
 * // 30
 * ```
 *
 * @since 4.0.0
 */
export declare function make<A>(g: Functors.Guardable<A>, ...values: A[]): T<A>;
/**
 * Creates a `TypedSequence.t<A>` starting from a `GuardableModule<A>`
 *
 * You can add initial values as arguments.
 *
 * If an argument does pass the guard check, the function will throw a TypeError
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * for (const x of TypedSequence.make(Num, 10, 20, 30)) {
 *    console.log(x)
 * }
 * // 10
 * // 20
 * // 30
 * ```
 *
 * @since 4.0.0
 */
export declare function make<A>(g: Functors.GuardableModule<A>, ...values: A[]): T<A>;
/**
 * Checks if the parameter `x` is a `TypedSequence.t<unknown>`
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Sequence, Num } from 'tiinvo'
 *
 * const a = TypedSequence.make(Num)
 * const b = Sequence.make()
 *
 * TypedSequence.guard(a)     // true
 * TypedSequence.guard(b)     // false
 * ```
 *
 * @param x the guarded value
 * @returns
 *  - true if x is a TypedSequence
 *  - false otherwise
 * @group Guardables
 * @since 4.0.0
 */
export declare const guard: (x: unknown) => x is T<unknown>;
/**
 * Adds an element to the end of the `Sequence.t<A>` without mutating the original one.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 *
 * TypedSequence.append(s0, 30)       // TypedSequence(10, 20, 30)
 * ```
 *
 * @param a the TypedSequence
 * @param b the value to append
 * @since 4.0.0
 */
export declare function append<A>(a: T<A>, b: A): T<A>;
/**
 * Returns a unary function which adds an element `a` to the end of the
 * `Sequence.t<A>` without mutating the original one.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 *
 * TypedSequence.append(30)(s0)       // TypedSequence(10, 20, 30)
 * ```
 *
 * @param a the value to append
 * @returns the unary function
 * @since 4.0.0
 */
export declare function append<A>(a: A): Fn.Unary<T<A>, T<A>>;
/**
 * Concatenates two `TypedSequence.t<A>` and `TypedSequence.t<b>` and return a new `TypedSequence.t<A>`.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 * const s1 = TypedSequence.make(Num, 30, 40)
 *
 * TypedSequence.concat(s0, s1)       // TypedSequence(10, 20, 30, 40)
 * ```
 *
 * @param a the first TypedSequence
 * @param a the last TypedSequence
 * @returns the concatenated TypedSequence
 * @since 4.0.0
 */
export declare function concat<a extends T<any>>(a: a, b: a): a;
/**
 * Returns a unary function which concatenates two `TypedSequence.t<A>`
 * and `TypedSequence.t<b>` and return a new `TypedSequence.t<A>`.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 * const s1 = TypedSequence.make(Num, 30, 40)
 *
 * TypedSequence.concat(s1)(s0)       // TypedSequence(10, 20, 30, 40)
 * ```
 *
 * @param a the first TypedSequence
 * @param a the last TypedSequence
 * @returns the concatenated TypedSequence
 * @since 4.0.0
 */
export declare function concat<a extends T<any>>(a: a): Fn.Unary<a, a>;
/**
 * Adds an element to the start of the `TypedSequence.t<A>` without mutating the original one.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 *
 * TypedSequence.prepend(s0, 30)       // TypedSequence.make(Num, 30, 10, 20)
 * TypedSequence.prepend(30)(s0)       // TypedSequence.make(Num, 30, 10, 20)
 * ```
 *
 * @param a the TypedSequence
 * @param b the value to prepend
 * @returns the new TypedSequence
 * @since 4.0.0
 */
export declare function prepend<A>(a: T<A>, b: A): T<A>;
/**
 * Returns a unary function which adds an element to the start of
 * the `TypedSequence.t<A>` without mutating the original one.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20)
 *
 * TypedSequence.prepend(30)(s0)       // TypedSequence.make(Num, 30, 10, 20)
 * ```
 *
 * @param a the value
 * @returns the unary function
 * @since 4.0.0
 */
export declare function prepend<A>(a: A): Fn.Unary<T<A>, T<A>>;
/**
 * Sorts and returns a new `TypedSequence.t<A>` values with a `Comparable<A>` or `ComparableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const a = TypedSequence.make(Num, 5, 3, 1, 4, 2)
 * const b = TypedSequence.sort(a, Num)
 *
 * b          // TypedSequence.make(Num, 1, 2, 3, 4, 5)
 * ```
 *
 * @template A the sequence type
 * @param a the sequence
 * @param mod the Comparable used to sort the sequence
 * @returns the sorted sequence
 * @group Sortables
 * @since 4.0.0
 */
export declare function sort<A>(a: T<A>, mod: Functors.Comparable<A> | Functors.ComparableModule<A>): T<A>;
/**
 * Returns a unary function which sorts and returns a new
 * `TypedSequence.t<A>` values with a `Comparable<A>` or `ComparableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sort = TypedSequence.sort(Num)
 *
 * const a = TypedSequence.make(Num, 5, 3, 1, 4, 2)
 * const b = sort(a)
 *
 * b          // TypedSequence.make(Num, 1, 2, 3, 4, 5)
 * ```
 *
 * @template A the sequence type
 * @param a the sequence
 * @returns the sorting unary function
 * @group Sortables
 * @since 4.0.0
 */
export declare function sort<A>(a: Functors.Comparable<A> | Functors.ComparableModule<A>): Fn.Unary<T<A>, T<A>>;
/**
 * Counts the number of elements that satisfy a given predicate
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s = TypedSequence.make(Num, 10, 20, 30)
 *
 * TypedSequence.count(s, Num.gt(10))   // 2
 * TypedSequence.count(Num.gt(10))(s)   // 2
 * ```
 *
 * @param a the sequence
 * @returns the counted elements
 * @since 4.0.0
 */
export declare const count: typeof Sequence.count;
/**
 *
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Str } from 'tiinvo'
 *
 * const s = TypedSequence.make(Str, 'hello', 'world')
 *
 * TypedSequence.get(s, 0)       // 'hello'
 * TypedSequence.get(s, 9)       // throws RangeError
 * ```
 *
 * @since 4.0.0
 */
export declare const get: typeof Sequence.get;
/**
 * Gets the first element of a TypedSequence.
 *
 * Returns `Option.None` if none is found.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20, 30)
 * const s1 = TypedSequence.make(Num)
 *
 * TypedSequence.first(s0)       // 10
 * TypedSequence.first(s1)       // null
 * ```
 *
 * @since 4.0.0
 */
export declare const first: <A>(t: Sequence.T<A>) => import("./Option.js").T<A>;
/**
 * Gets a Sequence.t<A>'s last element.
 *
 * Returns `Option.None` if none is found.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 10, 20, 30)
 * const s1 = TypedSequence.make(Num)
 *
 * TypedSequence.last(s0)       // 30
 * TypedSequence.last(s1)       // null
 * ```
 * ```
 *
 * @since 4.0.0
 */
export declare const last: <A>(t: Sequence.T<A>) => import("./Option.js").T<A>;
/**
 * Gets the length of a `TypedSequence.t<A>`
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s = TypedSequence.make(Num, 1, 2, 3)
 *
 * TypedSequence.length(s)           // 3
 * ```
 *
 * @since 4.0.0
 */
export declare const length: <A>(t: Sequence.T<A>) => number;
/**
 * Gets values of a `TypedSequence.t<A>` as an immutable indexed object.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Str } from 'tiinvo'
 *
 * const s = TypedSequence.make(Str, 'hello', 'world')
 *
 * TypedSequence.values(s)       // { 0: 'hello', 1: 'world' }
 * ```
 *
 * @since 4.0.0
 */
export declare const values: <A>(t: Sequence.T<A>) => Record<number, A>;
/**
 * Compares two `TypedSequence.t<A>` values with a `Comparable<A>` or `ComparableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 1, 2, 3)
 * const s1 = TypedSequence.make(Num, 1, 2, 3)
 *
 * TypedSequence.cmp(Num, s0, s1)       // 0
 * TypedSequence.cmp(Num)(s0, s1)       // 0
 * TypedSequence.cmp(Num, s0)(s1)       // 0
 * TypedSequence.cmp(Num)(s0)(s1)       // 0
 * ```
 *
 * @template A the sequence's element type
 *
 * @param cmp the `Comparable<A>` or `ComparableModule<A>` functor
 * @param a the first sequence
 * @param b the second sequence
 * @returns the comparison result
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function cmp<A>(cmp: Functors.Comparable<A> | Functors.ComparableModule<A>, a: T<A>, b: T<A>): Functors.ComparableResult;
/**
 * Returns a comparer function that compares two `TypedSequence.t<A>` values with a `Comparable<A>` or `ComparableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 1, 2, 3)
 * const s1 = TypedSequence.make(Num, 1, 2, 3)
 *
 * const c = TypedSequence.cmp(Num, s0)
 *
 * c(s1)       // 0
 * ```
 *
 * @template A the sequence's element type
 *
 * @param cmp the `Comparable<A>` or `ComparableModule<A>` functor
 * @param a the first sequence
 *
 * @returns the comparer function
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function cmp<A>(cmp: Functors.Comparable<A> | Functors.ComparableModule<A>, a: T<A>): Fn.Unary<T<A>, Functors.ComparableResult>;
/**
 * Returns a comparer function that compares two `TypedSequence.t<A>` values with a `Comparable<A>` or `ComparableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 1, 2, 3)
 * const s1 = TypedSequence.make(Num, 1, 2, 3)
 *
 * const c = TypedSequence.cmp(Num)
 *
 * c(s0, s1)       // 0
 * ```
 *
 * @template A the sequence's element type
 *
 * @param cmp the `Comparable<A>` or `ComparableModule<A>` functor
 *
 * @returns the comparer function
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function cmp<A>(cmp: Functors.Comparable<A> | Functors.ComparableModule<A>): Fn.Binary<T<A>, T<A>, Functors.ComparableResult>;
/**
 * Checks if two `TypedSequence.t<A>` values are equal with an `Equatable<A>` or `EquatableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 1, 2, 3)
 * const s1 = TypedSequence.make(Num, 1, 2, 3)
 *
 * TypedSequence.eq(Num, s0, s1)       // true
 * ```
 *
 * @template A the sequence's element type
 *
 * @param eq the `Equatable<A>` or `EquatableModule<A>` functor
 * @param a the first sequence
 * @param b the second sequence
 * @returns `true` if the sequences are equal, `false` otherwise
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function eq<A>(eq: Functors.Equatable<A> | Functors.EquatableModule<A>, a: T<A>, b: T<A>): boolean;
/**
 * Returns a comparer function that checks if two `TypedSequence.t<A>` values are equal with an `Equatable<A>` or `EquatableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s2 = TypedSequence.make(Num, 1, 2, 3)
 * const s3 = TypedSequence.make(Num, 1, 2, 3)
 *
 * const c = TypedSequence.eq(Num, s0)
 *
 * c(s1)       // true
 * ```
 *
 * @template A the sequence's element type
 *
 * @param eq the `Equatable<A>` or `EquatableModule<A>` functor
 * @param a the first sequence
 *
 * @returns the comparer function
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function eq<A>(eq: Functors.Equatable<A> | Functors.EquatableModule<A>, a: T<A>): Fn.Unary<T<A>, boolean>;
/**
 * Returns a comparer function that checks if two `TypedSequence.t<A>` values are equal with an `Equatable<A>` or `EquatableModule<A>` functor.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s0 = TypedSequence.make(Num, 1, 2, 3)
 * const s1 = TypedSequence.make(Num, 1, 2, 3)
 *
 * const c = TypedSequence.eq(Num)
 *
 * c(s0, s1)       // true
 * ```
 *
 * @template A the sequence's element type
 *
 * @param eq the `Equatable<A>` or `EquatableModule<A>` functor
 *
 * @returns the comparer function
 *
 * @group comparables
 * @since 4.0.0
 */
export declare function eq<A>(eq: Functors.Equatable<A> | Functors.EquatableModule<A>): Fn.Binary<T<A>, T<A>, boolean>;
/**
 * Returns `true` if the sorted list is empty.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s = TypedSequence.make(Num)
 * const s1 = TypedSequence.make(Num, 10)
 *
 * TypedSequence.empty(s)                // true
 * TypedSequence.empty(s1)               // false
 * ```
 *
 * @since 4.0.0
 */
export declare const empty: <A>(t: Sequence.T<A>) => boolean;
/**
 * Returns `true` if the sorted list is populated.
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const s = TypedSequence.make(Num, 10, 20, 30)
 *
 * TypedSequence.populated(s)                      // true
 * TypedSequence.populated(TypedSequence.make(Num))   // false
 * ```
 *
 * @since 4.0.0
 */
export declare const populated: <a>(t: Sequence.T<a>) => boolean;
/**
 * Converts a `TypedSequence.t<A>` to an array of `a[]`
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sl = TypedSequence.make(Num, 3, 2, 1)
 *
 * TypedSequence.toArray(sl)       // [3, 2, 1]
 * ```
 *
 * @param a the TypedSequence
 * @returns the array
 * @group Serializables
 * @since 4.0.0
 */
export declare const toArray: <A>(t: Sequence.T<A>) => A[];
/**
 * Converts a `TypedSequence.t<A>` to a jsonizable value
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sl = TypedSequence.make(Num, 3, 2, 1)
 *
 * TypedSequence.toJSON(sl)       // [3, 2, 1]
 * ```
 *
 * @param a the TypedSequence
 * @returns the JSON
 * @group Serializables
 * @since 4.0.0
 */
export declare const toJSON: <a>(t: Sequence.T<a>) => a[];
/**
 * Converts a `TypedSequence.t<A>` to a set of `Set<A>`
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sl = TypedSequence.make(Num, 3, 2, 1)
 *
 * TypedSequence.toMap(sl)      // Map([0, 3], [1, 2], [2, 1])
 * ```
 *
 * @param a the TypedSequence
 * @returns the Map
 * @group Serializables
 * @since 4.0.0
 */
export declare const toMap: <A>(t: Sequence.T<A>) => Map<string, A>;
/**
 * Converts a `TypedSequence.t<A>` to a set of `Set<A>`
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sl = TypedSequence.make(Num, 3, 2, 1)
 *
 * TypedSequence.toSet(sl)      // Set(3, 2, 1)
 * ```
 *
 * @param a the TypedSequence
 * @returns the Set
 * @group Serializables
 * @since 4.0.0
 */
export declare const toSet: <A>(t: Sequence.T<A>) => Set<A>;
/**
 * Converts a `TypedSequence.t<A>` to a string
 *
 * @example
 *
 * ```ts
 * import { TypedSequence, Num } from 'tiinvo'
 *
 * const sl = TypedSequence.make(Num, 3, 2, 1)
 *
 * TypedSequence.toString(sl)       // "3,2,1"
 * ```
 *
 * @param a the TypedSequence
 * @group Serializables
 * @since 4.0.0
 */
export declare const toString: <A>(t: Sequence.T<A>) => string;
export {};
